@@ -75,8 +75,10 @@ struct IsolateAllocations {
 /// and an optional zero copy buffer, each async Op is tied to a Promise in JavaScript.
 pub struct JsRuntime {
   // This is an Option<OwnedIsolate> instead of just OwnedIsolate to workaround
-  // an safety issue with SnapshotCreator. See JsRuntime::drop.
+  // a safety issue with SnapshotCreator. See JsRuntime::drop.
   v8_isolate: Option<v8::OwnedIsolate>,
+  // This is an Option<Box<JsRuntimeInspector> instead of just Box<JsRuntimeInspector>
+  // to workaround a safety issue. See JsRuntime::drop.
   inspector: Option<Box<JsRuntimeInspector>>,
   snapshot_creator: Option<v8::SnapshotCreator>,
   has_snapshotted: bool,
@@ -143,9 +145,9 @@ impl Drop for JsRuntime {
 fn v8_init(v8_platform: Option<v8::UniquePtr<v8::Platform>>) {
   // Include 10MB ICU data file.
   #[repr(C, align(16))]
-  struct IcuData([u8; 10413584]);
+  struct IcuData([u8; 10144432]);
   static ICU_DATA: IcuData = IcuData(*include_bytes!("icudtl.dat"));
-  v8::icu::set_common_data(&ICU_DATA.0).unwrap();
+  v8::icu::set_common_data_69(&ICU_DATA.0).unwrap();
 
   let v8_platform = v8_platform
     .unwrap_or_else(v8::new_default_platform)
@@ -204,9 +206,6 @@ pub struct RuntimeOptions {
   /// V8 platform instance to use. Used when Deno initializes V8
   /// (which it only does once), otherwise it's silenty dropped.
   pub v8_platform: Option<v8::UniquePtr<v8::Platform>>,
-
-  /// Create a V8 inspector and attach to the runtime.
-  pub attach_inspector: bool,
 }
 
 impl JsRuntime {
@@ -267,13 +266,8 @@ impl JsRuntime {
       (isolate, None)
     };
 
-    let maybe_inspector = if options.attach_inspector {
-      let inspector =
-        JsRuntimeInspector::new(&mut isolate, global_context.clone());
-      Some(inspector)
-    } else {
-      None
-    };
+    let inspector =
+      JsRuntimeInspector::new(&mut isolate, global_context.clone());
 
     let loader = options
       .module_loader
@@ -315,7 +309,7 @@ impl JsRuntime {
 
     let mut js_runtime = Self {
       v8_isolate: Some(isolate),
-      inspector: maybe_inspector,
+      inspector: Some(inspector),
       snapshot_creator: maybe_snapshot_creator,
       has_snapshotted: false,
       allocations: IsolateAllocations::default(),
@@ -346,8 +340,8 @@ impl JsRuntime {
     self.v8_isolate.as_mut().unwrap()
   }
 
-  pub fn inspector(&mut self) -> Option<&mut Box<JsRuntimeInspector>> {
-    self.inspector.as_mut()
+  pub fn inspector(&mut self) -> &mut Box<JsRuntimeInspector> {
+    self.inspector.as_mut().unwrap()
   }
 
   pub fn handle_scope(&mut self) -> v8::HandleScope {
@@ -386,7 +380,7 @@ impl JsRuntime {
       for (filename, source) in js_files {
         let source = source()?;
         // TODO(@AaronO): use JsRuntime::execute_static() here to move src off heap
-        self.execute(filename, &source)?;
+        self.execute_script(filename, &source)?;
       }
     }
     // Restore extensions
@@ -446,7 +440,9 @@ impl JsRuntime {
 
   /// Ensures core.js has the latest op-name to op-id mappings
   pub fn sync_ops_cache(&mut self) {
-    self.execute("<anon>", "Deno.core.syncOpsCache()").unwrap()
+    self
+      .execute_script("<anon>", "Deno.core.syncOpsCache()")
+      .unwrap()
   }
 
   /// Returns the runtime's op state, which can be used to maintain ops
@@ -457,23 +453,31 @@ impl JsRuntime {
     state.op_state.clone()
   }
 
-  /// Executes traditional JavaScript code (traditional = not ES modules)
+  /// Executes traditional JavaScript code (traditional = not ES modules).
   ///
   /// The execution takes place on the current global context, so it is possible
   /// to maintain local JS state and invoke this method multiple times.
   ///
+  /// `name` can be a filepath or any other string, eg.
+  ///
+  ///   - "/some/file/path.js"
+  ///   - "<anon>"
+  ///   - "[native code]"
+  ///
+  /// The same `name` value can be used for multiple executions.
+  ///
   /// `AnyError` can be downcast to a type that exposes additional information
   /// about the V8 exception. By default this type is `JsError`, however it may
   /// be a different type if `RuntimeOptions::js_error_create_fn` has been set.
-  pub fn execute(
+  pub fn execute_script(
     &mut self,
-    js_filename: &str,
-    js_source: &str,
+    name: &str,
+    source_code: &str,
   ) -> Result<(), AnyError> {
     let scope = &mut self.handle_scope();
 
-    let source = v8::String::new(scope, js_source).unwrap();
-    let name = v8::String::new(scope, js_filename).unwrap();
+    let source = v8::String::new(scope, source_code).unwrap();
+    let name = v8::String::new(scope, name).unwrap();
     let origin = bindings::script_origin(scope, name);
 
     let tc_scope = &mut v8::TryCatch::new(scope);
@@ -510,6 +514,8 @@ impl JsRuntime {
     // TODO(piscisaureus): The rusty_v8 type system should enforce this.
     state.borrow_mut().global_context.take();
 
+    self.inspector.take();
+
     // Overwrite existing ModuleMap to drop v8::Global handles
     self
       .v8_isolate()
@@ -608,8 +614,8 @@ impl JsRuntime {
     cx: &mut Context,
     wait_for_inspector: bool,
   ) -> Poll<Result<(), AnyError>> {
-    // We always poll the inspector if it exists.
-    let _ = self.inspector().map(|i| i.poll_unpin(cx));
+    // We always poll the inspector first
+    let _ = self.inspector().poll_unpin(cx);
 
     let state_rc = Self::state(self.v8_isolate());
     let module_map_rc = Self::module_map(self.v8_isolate());
@@ -1473,7 +1479,7 @@ pub mod tests {
     runtime.sync_ops_cache();
 
     runtime
-      .execute(
+      .execute_script(
         "setup.js",
         r#"
         function assert(cond) {
@@ -1492,7 +1498,7 @@ pub mod tests {
   fn test_dispatch() {
     let (mut runtime, dispatch_count) = setup(Mode::Async);
     runtime
-      .execute(
+      .execute_script(
         "filename.js",
         r#"
         let control = 42;
@@ -1511,7 +1517,7 @@ pub mod tests {
   fn test_dispatch_no_zero_copy_buf() {
     let (mut runtime, dispatch_count) = setup(Mode::AsyncZeroCopy(false));
     runtime
-      .execute(
+      .execute_script(
         "filename.js",
         r#"
         Deno.core.opAsync("op_test");
@@ -1525,7 +1531,7 @@ pub mod tests {
   fn test_dispatch_stack_zero_copy_bufs() {
     let (mut runtime, dispatch_count) = setup(Mode::AsyncZeroCopy(true));
     runtime
-      .execute(
+      .execute_script(
         "filename.js",
         r#"
         let zero_copy_a = new Uint8Array([0]);
@@ -1553,7 +1559,7 @@ pub mod tests {
     });
 
     // Rn an infinite loop, which should be terminated.
-    match isolate.execute("infinite_loop.js", "for(;;) {}") {
+    match isolate.execute_script("infinite_loop.js", "for(;;) {}") {
       Ok(_) => panic!("execution should be terminated"),
       Err(e) => {
         assert_eq!(e.to_string(), "Uncaught Error: execution terminated")
@@ -1567,7 +1573,7 @@ pub mod tests {
 
     // Verify that the isolate usable again.
     isolate
-      .execute("simple.js", "1 + 1")
+      .execute_script("simple.js", "1 + 1")
       .expect("execution should be possible again");
 
     terminator_thread.join().unwrap();
@@ -1592,7 +1598,7 @@ pub mod tests {
     run_in_task(|mut cx| {
       let (mut runtime, _dispatch_count) = setup(Mode::Async);
       runtime
-        .execute(
+        .execute_script(
           "bad_op_id.js",
           r#"
           let thrown;
@@ -1615,7 +1621,7 @@ pub mod tests {
   fn syntax_error() {
     let mut runtime = JsRuntime::new(Default::default());
     let src = "hocuspocus(";
-    let r = runtime.execute("i.js", src);
+    let r = runtime.execute_script("i.js", src);
     let e = r.unwrap_err();
     let js_error = e.downcast::<JsError>().unwrap();
     assert_eq!(js_error.end_column, Some(11));
@@ -1626,7 +1632,7 @@ pub mod tests {
     run_in_task(|mut cx| {
       let (mut runtime, _dispatch_count) = setup(Mode::Async);
       runtime
-        .execute(
+        .execute_script(
           "encode_decode_test.js",
           include_str!("encode_decode_test.js"),
         )
@@ -1642,7 +1648,7 @@ pub mod tests {
     run_in_task(|mut cx| {
       let (mut runtime, _dispatch_count) = setup(Mode::Async);
       runtime
-        .execute(
+        .execute_script(
           "serialize_deserialize_test.js",
           include_str!("serialize_deserialize_test.js"),
         )
@@ -1675,7 +1681,7 @@ pub mod tests {
       runtime.register_op("op_err", op_sync(op_err));
       runtime.sync_ops_cache();
       runtime
-        .execute(
+        .execute_script(
           "error_builder_test.js",
           include_str!("error_builder_test.js"),
         )
@@ -1693,7 +1699,7 @@ pub mod tests {
         will_snapshot: true,
         ..Default::default()
       });
-      runtime.execute("a.js", "a = 1 + 2").unwrap();
+      runtime.execute_script("a.js", "a = 1 + 2").unwrap();
       runtime.snapshot()
     };
 
@@ -1703,7 +1709,7 @@ pub mod tests {
       ..Default::default()
     });
     runtime2
-      .execute("check.js", "if (a != 3) throw Error('x')")
+      .execute_script("check.js", "if (a != 3) throw Error('x')")
       .unwrap();
   }
 
@@ -1714,7 +1720,7 @@ pub mod tests {
         will_snapshot: true,
         ..Default::default()
       });
-      runtime.execute("a.js", "a = 1 + 2").unwrap();
+      runtime.execute_script("a.js", "a = 1 + 2").unwrap();
       let snap: &[u8] = &*runtime.snapshot();
       Vec::from(snap).into_boxed_slice()
     };
@@ -1725,7 +1731,7 @@ pub mod tests {
       ..Default::default()
     });
     runtime2
-      .execute("check.js", "if (a != 3) throw Error('x')")
+      .execute_script("check.js", "if (a != 3) throw Error('x')")
       .unwrap();
   }
 
@@ -1749,7 +1755,7 @@ pub mod tests {
       },
     );
     let err = runtime
-      .execute(
+      .execute_script(
         "script name",
         r#"let s = ""; while(true) { s += "Hello"; }"#,
       )
@@ -1801,7 +1807,7 @@ pub mod tests {
     );
 
     let err = runtime
-      .execute(
+      .execute_script(
         "script name",
         r#"let s = ""; while(true) { s += "Hello"; }"#,
       )
@@ -1869,7 +1875,7 @@ pub mod tests {
   fn test_error_without_stack() {
     let mut runtime = JsRuntime::new(RuntimeOptions::default());
     // SyntaxError
-    let result = runtime.execute(
+    let result = runtime.execute_script(
       "error_without_stack.js",
       r#"
 function main() {
@@ -1887,7 +1893,7 @@ main();
   #[test]
   fn test_error_stack() {
     let mut runtime = JsRuntime::new(RuntimeOptions::default());
-    let result = runtime.execute(
+    let result = runtime.execute_script(
       "error_stack.js",
       r#"
 function assert(cond) {
@@ -1915,7 +1921,7 @@ main();
     run_in_task(|cx| {
       let mut runtime = JsRuntime::new(RuntimeOptions::default());
       runtime
-        .execute(
+        .execute_script(
           "error_async_stack.js",
           r#"
 (async () => {
@@ -1953,7 +1959,7 @@ main();
     let mut runtime = JsRuntime::new(RuntimeOptions::default());
     // Call non-existent op so we get error from `core.js`
     let error = runtime
-      .execute(
+      .execute_script(
         "core_js_stack_frame.js",
         "Deno.core.opSync('non_existent');",
       )
@@ -1970,6 +1976,6 @@ main();
       ..Default::default()
     };
     let mut runtime = JsRuntime::new(options);
-    runtime.execute("<none>", "").unwrap();
+    runtime.execute_script("<none>", "").unwrap();
   }
 }
